`define NOAC 2'b00
`define DLTILE 2'b01
`define DLSPRITE 2'b10
`define SETTMAP 2'b11

module data_transfer(clk, rst_n, PORT_A, PORT_B, PORT_C, PORT_D
					, tRAM_wren, tRAM_data, tRAM_addr
					, spRAM_wren, spRAM_data, spRAM_addr
					, scRAM_wren, scRAM_data, scRAM_addr);

input clk;
input rst_n;
input [7:0] PORT_A;
input [7:0] PORT_B;
input [7:0] PORT_C;
input [7:0] PORT_D;

//outputs to control tile RAM
output reg tRAM_wren;
output reg [7:0] tRAM_data;
output reg [9:0] tRAM_addr;
//end outputs to control tile RAM

//outputs to control sprite RAM
output reg spRAM_wren;
output reg [7:0] spRAM_data;
output reg [7:0] spRAM_addr;
//end outputs to control sprite RAM

//output to control screen RAM

output reg scRAM_wren;
output reg [3:0] scRAM_data;
output reg [8:0] scRAM_addr;

//end outputs to control screen RAM

wire [1:0] mode;

wire DCL;		//data clock for coordinated transfer from ATMEL
reg last_DCL;
wire posedge_DCL;


wire [4:0] X_coord;
wire [4:0] Y_coord;
wire [3:0] tile_id_to_dl;
wire [3:0] tile_id_for_map;
wire [1:0] sprite_id;
wire [5:0] address;
wire [7:0] pixeldata;

assign mode = PORT_D[7:6];
assign X_coord = PORT_A[4:0];
assign Y_coord = PORT_B[4:0];

assign pixeldata = PORT_A[7:0];
assign tile_id_to_dl = PORT_B[3:0];
assign tile_id_for_map = PORT_C[3:0];
assign address = PORT_C[5:0];
assign sprite_id = PORT_B[1:0];

assign DCL = PORT_D[0];
assign posedge_DCL = DCL & ~last_DCL;


always @(posedge clk)
if (!rst_n)
begin
	last_DCL <= 0;
end
else
begin
	last_DCL <= DCL;
end


always @(posedge clk)
if (!rst_n)
begin
	tRAM_wren <= 1'b0;
	tRAM_addr <= 10'b0;
	tRAM_data <= 8'b0;

	spRAM_wren <= 1'b0;
	spRAM_addr <= 8'b0;
	spRAM_data <= 8'b0;


	scRAM_wren <= 1'b0;
	scRAM_addr <= 9'b0;
	scRAM_data <= 4'b0;

end
else
begin

	if (posedge_DCL == 1'b1)
	begin
		if (mode == `SETTMAP)
		begin
			scRAM_wren <= 1'b1;
			scRAM_addr <= 9'd26 * Y_coord + X_coord;
			scRAM_data[3:0] <= tile_id_for_map[3:0];
		end
		else if (mode == `DLSPRITE)
		begin
			spRAM_wren <= 1'b1;	
			spRAM_addr[7:0] <= {sprite_id[1:0],address[5:0]};
			spRAM_data[7:0] <= pixeldata[7:0];
		end
		else if (mode == `DLTILE)
		begin
			tRAM_wren <= 1'b1;
			tRAM_addr[9:0] <= {tile_id_to_dl[3:0], address[5:0]};
			tRAM_data[7:0] <= pixeldata[7:0];
		end
		else
		begin
			scRAM_wren <= 1'b0;
			tRAM_wren <= 1'b0;
			spRAM_wren <= 1'b0;
		end
	end
	else
	begin
		tRAM_wren <= 1'b0;
		spRAM_wren <= 1'b0;
		scRAM_wren <= 1'b0;
	end


end

endmodule
