module sprite_draw(clk, rst_n, en, sprite1X, sprite2X, sprite1ID, sprite2ID, row, col, sprite_addr, drawsprite);


input clk;
input rst_n;

input en;	//enable

input [7:0] sprite1X;
input [7:0] sprite2X;

input [1:0] sprite1ID;
input [1:0] sprite2ID;

input [7:0] row;

input [7:0] col;

output reg [7:0] sprite_addr;
output drawsprite;

reg drawsprite1, drawsprite2;

assign drawsprite = drawsprite1 | drawsprite2;

reg [7:0] sprite1X_last;
reg [7:0] sprite2X_last;

reg [7:0] sprite1toprow;
reg [7:0] sprite2toprow;

reg [2:0] sprow1;	
reg [2:0] spcol1;	
reg [2:0] sprow2;
reg [2:0] spcol2;


always @(posedge clk)
if (!rst_n)
begin
	sprow1 <= 3'b0;
	sprow2 <= 3'b0;
	spcol1 <= 3'b0;
	spcol2 <= 3'b0;
end
else
begin
	sprow1 <= row - sprite1toprow;		//we can afford a delay of one clock for the row
	sprow2 <= row - sprite2toprow;
	
	spcol1 <= col + 8'd7 - sprite1X; 
	spcol2 <= col + 8'd7 - sprite2X;
	
end


always @(posedge clk)
if (!rst_n)
begin
	sprite1X_last <= 8'd0;
	sprite2X_last <= 8'd0;
end
else
begin
	sprite1X_last <= sprite1X;
	sprite2X_last <= sprite2X;
end

always @(posedge clk)
if (!rst_n)
begin
	drawsprite1 <= 1'b0;
	drawsprite2 <= 0;
	sprite_addr <= 8'b0;
	sprite1toprow <= 8'b0;
	sprite2toprow <= 8'b0;
end
else if (en == 1'b1)
begin

//figure out address for sprite 2
if (sprite2X != 8'b11111111)
	begin
		if (sprite2X_last == 8'b11111111)
		begin
			sprite2toprow <= row;
			drawsprite2 <= 1'b0;
		end
		else
		begin
			if (row >= sprite2toprow && row < sprite2toprow + 8 && col+7 >= sprite2X && col < sprite2X +1)
			begin
				drawsprite2 <= 1'b1;
			
				sprite_addr <= {sprite2ID[1:0], sprow2[2:0], spcol2[2:0]};
			end
			else
			begin
				drawsprite2<= 1'b0;
			end
		end
	end
	
//if sprite 1 is active and in current pixel, it trumps sprite 2
	if (sprite1X != 8'b11111111)
	begin
		if (sprite1X_last == 8'b11111111)
		begin
			sprite1toprow <= row;
			drawsprite1 <= 1'b0;
		end
		else
		begin
			if (row >= sprite1toprow && row < sprite1toprow + 8 && col+8'd7 >= sprite1X && col < sprite1X  +1 )
			begin
				drawsprite1 <= 1'b1;
				
				sprite_addr <= {sprite1ID[1:0], sprow1[2:0], spcol1[2:0]};
			end
			else
			begin
				drawsprite1 <= 1'b0;
			end
		end
	end

//if neither sprite is activated, disable drawing of sprites, and reset sprite top rows to 0
	if (sprite1X == 8'b11111111 && sprite2X == 8'b11111111)
	begin
		drawsprite1 <= 1'b0;
		drawsprite2 <= 1'b0;
		sprite1toprow <= 0;
		sprite2toprow <= 0;
	end
end
endmodule
