//we want a black border on the left so that the number of active pixels
//in a line is a multiple of 200, our logical screen width

`define HFPORCH_END 10'd5				/*10'd16*/
`define HSYNC_END 	10'd37				/*10'd112*/
`define HBPORCH_END 10'd67				/*10'd200*/
`define HACTIVE_END 10'd267				/*10'd800*/

`define VFPORCH_END 10'd12
`define VSYNC_END 10'd14
`define VBPORCH_END 10'd45
`define VACTIVE_END 10'd525

`define sFPORCH 2'd0
`define sSYNC 2'd1
`define sBPORCH 2'd2
`define sACTIVE 2'd3

module vga_out(clk, rst_n, hsync, vsync, frame_int, line_int, RAM_to_screen, row, col);

input clk;
input rst_n;

output reg hsync;
output reg vsync;
output reg frame_int;
output reg line_int;

output reg RAM_to_screen;

output reg [7:0] row;
output reg [7:0] col;

reg [1:0] rowdivcounter;

reg [9:0] hcounter;
reg [9:0] vcounter;

wire [9:0] vcounter_plus_1;

reg [1:0] hstate;
reg [1:0] vstate;

reg [1:0] last_hstate;
reg [1:0] last_vstate;

assign vcounter_plus_1 = vcounter + 10'b1;

always @(posedge clk)
begin
	if (hcounter >= `HBPORCH_END)
		hstate <= `sACTIVE;
	else if (hcounter >= `HSYNC_END)
		hstate <= `sBPORCH;
	else if (hcounter >= `HFPORCH_END)
		hstate <= `sSYNC;
	else
		hstate <= `sFPORCH;

	if (vcounter >= `VBPORCH_END)
		vstate <= `sACTIVE;
	else if (vcounter >= `VSYNC_END)
		vstate <= `sBPORCH;
	else if (vcounter >= `VFPORCH_END)
		vstate <= `sSYNC;
	else
		vstate <= `sFPORCH;
end

always @(posedge clk)
if (!rst_n)
begin
	row <= 0;
	col <= 0;
	rowdivcounter <= 0;
end
else
begin
	if (vstate == `sACTIVE)
	begin
		if (hstate == `sACTIVE)
		begin
			col <= col + 8'd1;
		end	
		else
		begin
			if (col != 0)
			begin
				if (rowdivcounter != 2'd2)
				begin
					rowdivcounter <= rowdivcounter + 2'd1;
				end
				else
				begin
					rowdivcounter <= 2'd0;
					row <= row+8'd1;
				end
			end
			col <= 8'd0;
		end
	end
	else
	begin
		row <= 8'd0;
		rowdivcounter <= 2'd0;
		col <= 8'd0;
	end
end

always @(posedge clk)
if (!rst_n)
begin
	last_hstate <= 1'b0;
	last_vstate <= 1'b0;
end
else
begin
	last_hstate <= hstate;
	last_vstate <= vstate;
end


always @(posedge clk)
if (rst_n == 1'b0)
begin
	hsync <= 1'b0;
	vsync <= 1'b0;
	frame_int <= 1'b0;
	line_int <= 1'b0;
	hcounter <= 10'b0;
	vcounter <= 10'b0;
	RAM_to_screen <= 1'b0;
end
else
begin


//generate hsync pulse (0 during sync, 1 otherwise)
if (hstate == `sSYNC)
	hsync <= 1'b0;
else
	hsync <= 1'b1;


//generate vsync pulse (0 during sync, 1 otherwise)
if (vstate == `sSYNC)
	vsync <= 1'b0;
else
	vsync <= 1'b1;


//generate line interrupt
//this will generate interrupts even during vertical blanking.  Do we want that?

if (hcounter <= 13 && hcounter >=10)
	line_int <= 0;
else
	line_int <= 1;


//generate frame interrupt
//if the line interrupt is allowed to happen during vertical blanking,
//both interrupts will trigger at the same time when the frame begins. Do we want that?
if (vcounter == 0 && hcounter <= 3)
	frame_int <= 0;
else
	frame_int <= 1;


//tell top level that RAMs should be read to generate output
if (hstate == `sACTIVE && vstate == `sACTIVE)
begin
	RAM_to_screen <= 1'b1;
end
else
	RAM_to_screen <= 1'b0;

//update counters
	
	hcounter <= hcounter + 10'd1;
	if (hcounter+1 >= `HACTIVE_END)
	begin
		hcounter <= 0;
		vcounter <= vcounter_plus_1;
		if (vcounter_plus_1 >= `VACTIVE_END)
			vcounter <= 0;
	end
	
end

endmodule
